# ----------------------------------
# ==================================
# ----------------------------------
#
# "Convert objects randomly" script
# Released into the public domain by Agamemnus.
# Version 1.1: November 12, 2009
# Contact the author: sketchup at flyingsoft dot phatcode dot net
#
# Purpose:
#
#     Easily converts a set of objects (target) into another set (base) randomly,
# distributed amongst members of the base set. Similarly changes colors.
#
# Use:
# 1)  Select a set of objects -- currently only a set of components and/or groups work.
# 1b) Pick "Select base objects" in "Plugins">"Convert Objects Randomly".
# 2)  Select a target set of objects -- currently only components, groups, or lines.
# 2b) Pick "Convert selected to random base objects" in "Plugins">"Choose targets (object randomization)".
# 2c) Done! You may pick "Choose targets (object randomization)" again if you wish --
#     your previous base and target object sets apply if you did not fundamentally change
#     (explode, convert to group, etc-- geometry editing is fine) the target components.
# 3) Set "Plugins">"Choose targets (object randomization)">"Settings">"Convert lines to base objects?" to "YES"
#    if you want ungrouped lines in the target set to be affected. Set "NO" otherwise.
#    The default is "YES".
# 4) For color randomization, the same method is used, except that nothing is converted into
#    groups/component instances, and any object (line, group, component) may be used as a target
#    and as a base.
#
# Behavior of result objects in version 1:
# If the picked "base" object is a group, the target object is converted into a unique component.
# If the picked "base" object is a component, the target object is converted into a component
# instance of that component type.
#
# 
# ==Miscellaneous Notes:==
# 1.) Much thanks to the Sketchup forum regulars in helping with problems in this script.
#
# 2.) Looking for Work :(
# ...in the fields of:
# economic analysis
# business strategy
# marketing
# UI design
# web based programming
# 2D visual art
# 3D visual art.
# general programming.
# M.A. in International Economics and Finance at Brandeis IBS, BA in economics at Brandeis IBS and Brandeis University.
# 16 years of experience in amateur programming in BASIC-based languages (Basic, Freebasic, VB, etc.)
# Several years of experience in PHP and Javascript. Experience in C-LISP, Matlab, and "some"
# experience in Ruby (eh!) and C.
# Work environment must at a _minimum_ contain penguins performing tricks, live.
#
# ----------------------------------
# ==================================
# ----------------------------------

require 'sketchup.rb'

module Convert_objects_randomly

 @base_entities = []
 @target_entities = []
 @convert_lines = true
 @base_entities_colors = []

 def Convert_objects_randomly.choosebase
  @base_entities = Sketchup.active_model.selection.to_a
  if @base_entities.empty?
   results = UI.messagebox "Please select base groups and/or components first."
   Sketchup.send_action "selectSelectionTool:"
   return
  end
 end



 def Convert_objects_randomly.choosetarget_changeobjects
  if @base_entities.empty?
   results = UI.messagebox "Please select base groups and/or components first."
   Sketchup.send_action "selectSelectionTool:"
   return
  end
  target_entities = Sketchup.active_model.selection.to_a
  if target_entities.empty?
   if @target_entities.empty?
    results = UI.messagebox "Please select target groups and/or components first."
    Sketchup.send_action "selectSelectionTool:"
   return
   end
   target_entities = @target_entities
  end
  target_entities.each do |current_entity|
   if current_entity.is_a?(Sketchup::Group)
    current_entity = current_entity.to_component
    current_entity.definition = @base_entities[rand(@base_entities.length)].definition
   elsif current_entity.is_a?(Sketchup::ComponentInstance)
    current_entity = current_entity.make_unique
    current_entity.definition = @base_entities[rand(@base_entities.length)].definition
   elsif (@convert_lines == true)
    current_entity = Sketchup.active_model.active_entities.add_group(current_entity)
    current_entity = current_entity.to_component
    current_entity.definition = @base_entities[rand(@base_entities.length)].definition
   end 
  end

  @target_entities = target_entities
  Sketchup.active_model.commit_operation
 end



 def Convert_objects_randomly.settings
  if (@convert_lines == true)
   default = "yes"
  else
   default = "no"
  end
  settings_query1 = inputbox(["Convert lines to base objects? "],[default], ["yes|no"], "Convert Objects Randomly Settings")
  if (settings_query1[0] == "yes")
   @convert_lines = true
  else
   @convert_lines = false
  end
 end



 def Convert_objects_randomly.choosebasecolors
  @base_entities_colors = Sketchup.active_model.selection.to_a
  if @base_entities_colors.empty?
   results = UI.messagebox "Please select base groups and/or components first."
   Sketchup.send_action "selectSelectionTool:"
   return
  end
  for i in (0 .. @base_entities_colors.length-1)
   @base_entities_colors[i] = @base_entities_colors[i].material
  end
 end





 def Convert_objects_randomly.choosebasecolors
  @base_entities_colors = Sketchup.active_model.selection.to_a
  if @base_entities_colors.empty?
   results = UI.messagebox "Please select base groups and/or components first."
   Sketchup.send_action "selectSelectionTool:"
   return
  end
  for i in (0 .. @base_entities_colors.length-1)
   @base_entities_colors[i] = @base_entities_colors[i].material
  end
 end



 def Convert_objects_randomly.choosetarget_changecolors
  if @base_entities_colors.empty?
   results = UI.messagebox "Please select base groups and/or components first."
   Sketchup.send_action "selectSelectionTool:"
   return
  end

  target_entities = Sketchup.active_model.selection.to_a
  if target_entities.empty?
   if @target_entities.empty?
    results = UI.messagebox "Please select target groups and/or components first."
    Sketchup.send_action "selectSelectionTool:"
    return
   end
   target_entities = @target_entities
  end
   
  target_entities.each do |current_entity|
   current_entity.material = @base_entities_colors[rand(@base_entities_colors.length)]
  end

  @target_entities = target_entities
  Sketchup.active_model.commit_operation
 end


end #Convert_objects_randomly module


Convert_objects_randomly_menu = UI.menu("Plugins").add_submenu("Convert Objects Randomly")
Convert_objects_randomly_menu.add_item("Select base objects") {Convert_objects_randomly.choosebase}
Convert_objects_randomly_menu.add_item("Select base colors") {Convert_objects_randomly.choosebasecolors}
Convert_objects_randomly_menu.add_item("Choose targets (object randomization)") {Convert_objects_randomly.choosetarget_changeobjects}
Convert_objects_randomly_menu.add_item("Choose targets (color randomization)") {Convert_objects_randomly.choosetarget_changecolors}
Convert_objects_randomly_menu.add_item("Settings") {Convert_objects_randomly.settings}